flutter_quick_db library
Classes
- $DatabaseWrapper
- An simple wrapper for Sembast's Database, allowing consumers of this package to name their database classes "Database" without name clashes
- Boundary
- Sort order boundary, lower or upper to use in a Finder
- Database
- Database.
- File
- A reference to a file on the file system.
- Filter
- Filter for searching into the database
- Finder
- Finder helper for searching a given store
-
IntStore<
T extends IntStoreModel> -
A
_DataStorewhose ids/keys are integers - IntStoreModel
- QuickDatabase
-
SortOrder<
T extends Object?> - Sort order
-
StringStore<
T extends StringStoreModel> -
A
_DataStorewhose ids/keys are strings - StringStoreModel
Properties
- databaseFactoryIo → DatabaseFactory
-
Factory for io databases (flutter, dart vm).
no setter
Functions
-
buildDb(
BuilderOptions options) → Builder